@@ -1,6 +1,6 @@
 /*
  * synergy -- mouse and keyboard sharing utility
- * Copyright (C) 2013-2016 Symless Ltd.
+ * Copyright (C) 2015-2016 Symless Ltd.
  * 
  * This package is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,24 +17,22 @@
 
 #pragma once
 
-#include "server/ClientProxy1_4.h"
-#include "base/Stopwatch.h"
-#include "common/stdvector.h"
+#include "server/ClientProxy1_5.h"
 
 class Server;
 class IEventQueue;
 
-//! Proxy for client implementing protocol version 1.5
-class ClientProxy1_5 : public ClientProxy1_4 {
+//! Proxy for client implementing protocol version 1.6
+class ClientProxy1_6 : public ClientProxy1_5 {
 public:
-    ClientProxy1_5(const String& name, synergy::IStream* adoptedStream, Server* server, IEventQueue* events);
-    ~ClientProxy1_5();
+    ClientProxy1_6(const String& name, synergy::IStream* adoptedStream, Server* server, IEventQueue* events);
+    ~ClientProxy1_6();
 
-    virtual void        sendDragInfo(UInt32 fileCount, const char* info, size_t size);
-    virtual void        fileChunkSending(UInt8 mark, char* data, size_t dataSize);
-    virtual bool        parseMessage(const UInt8* code);
-    void                fileChunkReceived();
-    void                dragInfoReceived();
+    virtual void        setClipboard(ClipboardID id, const IClipboard* clipboard);
+    virtual bool        recvClipboard();
+
+private:
+    void                handleClipboardSendingEvent(const Event&, void*);
 
 private:
     IEventQueue*        m_events;
